package com.sumbytes.helloblog.auth.controller;

import com.sumbytes.common.annotation.LoginRequired;
import com.sumbytes.common.base.domain.Result;
import com.sumbytes.common.enums.ErrorEnum;
import com.sumbytes.common.util.ExceptionUtil;
import com.sumbytes.helloblog.auth.domain.vo.AuthUserSocialVO;
import com.sumbytes.helloblog.auth.service.AuthUserSocialService;
import com.sumbytes.system.enums.RoleEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping("/social")
public class AuthSocialController {

    @Autowired
    private AuthUserSocialService authUserSocialService;

    @PostMapping("/social/v1/add")
    @LoginRequired(role = RoleEnum.ADMIN)
    public Result saveSocial(@RequestBody AuthUserSocialVO authUserSocialVO) {
        ExceptionUtil.isRollback(StringUtils.isBlank(authUserSocialVO.getCode()), ErrorEnum.PARAM_ERROR);
        return authUserSocialService.saveAuthUserSocial(authUserSocialVO);
    }

    @PutMapping("/social/v1/update")
    @LoginRequired(role = RoleEnum.ADMIN)
    public Result editSocial(@RequestBody AuthUserSocialVO authUserSocialVO) {
        ExceptionUtil.isRollback(authUserSocialVO.getId() == null, ErrorEnum.PARAM_ERROR);
        return authUserSocialService.editAuthUserSocial(authUserSocialVO);
    }

    @GetMapping("/social/v1/{id}")
    public Result getSocial(@PathVariable("id") Long id) {
        return authUserSocialService.getSocial(id);
    }

    @DeleteMapping("/social/v1/{id}")
    public Result deleteSocial(@PathVariable("id") Long id) {
        return authUserSocialService.deleteSocial(id);
    }

    @LoginRequired(role = RoleEnum.ADMIN)
    @GetMapping("/social/v1/list")
    public Result getSocialList(AuthUserSocialVO authUserSocialVO) {
        return authUserSocialService.getSocialList(authUserSocialVO);
    }

    @GetMapping("/social/v1/socials")
    public Result getSocialEnableList(AuthUserSocialVO authUserSocialVO) {
        authUserSocialVO.setIsEnabled(1);
        return authUserSocialService.getSocialList(authUserSocialVO);
    }

    @GetMapping("/social/v1/info")
    public Result getSocialInfo() {
        return authUserSocialService.getSocialInfo();
    }
}
